/**
 * The purpose of this program is to display words in a
 * persistence of vision POV LED Sweep on the QuickStart LED.
 *
 * A POV LED Sweep is where you move the QuickStart from
 * left to right while holding the "Show" button.
 * If you sweep the QuickStart at the right speed, then
 * you should be able to see letters printed in mid air.
 *
 * Copyright 2011 by Steve Denson.
 * Rights MIT Licensed. See end of file.
 */
 
/*
 * User Instructions: choose a phrase by setting the 4 left most LEDs
 * to one of 16 combinations. Press the right most LED to start
 * the POV message phrase. Other LEDs can be used to control the
 * message rate or select a random phrase.
 *
 * [   =    =    =    = ,  =    = ,  = ,  =   ]
 * [   *    *    *    * ,  * ,  * ,  * ,  *   ]
 * [   Phrase Number   , Rate,Incr,Rest,Show  ]
 *
 * Phrase Number is index of the phrase to display.
 * Rate is an display speed increment that cycles from slow to fast.
 * Incr is an Phrase Number increment that cycles 0 to 15.
 * Rest sets Phrase Number 0.
 * Show starts the POV LED Sweep to display the message.
 *
 */

/*
 * Build and Load Instructions:
 *
 * For LMM ... total program size 4476 bytes.
 * $ propeller-elf-gcc -o main.elf -Os main.c
 * $ propeller-load -r main.elf -p <port number>
 *
 * For XMMC ... not necessary, but it works.
 * $ propeller-elf-gcc -o main.elf -Os mxmmc main.c
 * $ propeller-load -r main.elf -p <port number> -b eeprom
 *
 */

/* Set this to 1 to see output on console. Makes program bigger. */
#define ENABLE_DEBUG 0

#if ENABLE_DEBUG
#include <stdio.h>
#endif
#include <propeller.h>

/* Font table */
#define FONTCHARS 128
#define FONTWIDTH 8
char fonttab[FONTCHARS][FONTWIDTH];

/* constants for buttons and lights */

#define SHOW_BUTTON 0
#define REST_BUTTON 1
#define INCR_BUTTON 2
#define RATE_BUTTON 3
#define PHRASE_LED  4

/* Forward declarations of functions */

void    LEDOUT(int val);                // LED output function
void    usleep(int ms);                 // microsecond sleep
void    msleep(int ms);                 // millisecond sleep
int     getButton(int n);               // used to get button states
int     getButtons();                   // used to get button states
void    message(int phrase, int rate);  // show POV LED Sweep

// Test Phrase  "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ!.?:)\0",

/* Random words */
char *phrase[] =
{
    "OK\0",
    "Test\0",
    "Hello\0",
    "Bye!\0",

    "Thanks\0",
    "KissIt\0",
    "OnTime\0",
    "Bigger\0",

    "Smaller\0",
    "GoTeam\0",
    "Never\0",
    "Grease!\0",

    "Who?\0",
    "Waah!\0",
    "DoIt!\0",
    "Good Luck\0"
};

/*
 * This is the main function.
 * It decodes the user interface and shows messages.
 */
int main()
{
    int index = 0;
    int butns = 0;
    int delay = 1;

    DIRA |= (1<<30);                    // make sure quickstart tx is driven

#if ENABLE_DEBUG
    printf("\nPOV LED Sweep Demo\n");
#endif

    /*
     * This is the main program loop.
     * It decodes the buttons,
     * displays message index,
     * and calls the POV Message sweep.
     */
    while(1)
    {
        butns = getButtons();           // get button state

        if(butns & (1<<REST_BUTTON))    // Reset?
        {
            index = 0;
            LEDOUT(1<<REST_BUTTON);
            msleep(200);
        }
        else
        if(butns & (1<<INCR_BUTTON))    // Increment?
        {
            index++;
            index &= 0xF;
            LEDOUT(1<<INCR_BUTTON);
            msleep(200);
        }
        else
        if(butns & (1<<RATE_BUTTON))    // Change Rate?
        {
            delay++;
            delay &= 0x7;
            if(!delay) 
            {
                delay = 1;
                LEDOUT(1<<RATE_BUTTON);
                msleep(100);
                LEDOUT(0);
                msleep(100);
            }
            LEDOUT(1<<RATE_BUTTON);
            msleep(200);
        }

        LEDOUT(index<<PHRASE_LED);      // Show current phrase number
        msleep(200);


        if(butns & (1<<SHOW_BUTTON))    // Display LED POV message
        {
            message(index, delay);
            msleep(300);
        }
        msleep(200);
    }
    return 0;
}

/*
 * Set the LEDs to val
 */
void LEDOUT(int val)
{
#if ENABLE_DEBUG
    int n;
    putchar('\n');
    for(n = 7; n > -1; n--)
    {
        if(val & (1<<n))
            putchar('#');
        else
            putchar(' ');
    }
#endif
    DIRA |= 0xff << 16;
    OUTA = (OUTA & ~(0xff<<16)) | (val<<16);
}

/*
 * We use the hubtext attribute to make sure functions
 * go into hub memory even with xmm code.
 */
#if defined(__PROPELLER__) && defined(__GNUC__)
#define HUBCODE __attribute__((section(".hubtext")))
#else
#define HUBCODE
#endif

/*
 * msleep makes the cog wait for t milliseconds.
 */
HUBCODE
void msleep(int t)
{
    waitcnt((CLKFREQ/1000)*t+CNT);
}

HUBCODE
void usleep(int t)
{
    if(t < 10)  // very small t values will cause a hang
        return; // don't bother function delay is likely enough
    waitcnt((CLKFREQ/1000000)*t+CNT);
}

/*
 * Get a specific button number.
 * buttons are enumerated from 0 to 7.
 */
int getButton(int n)
{
    return (getButtons() & (1<<n));
}

/*
 * Get all buttons at once
 * Run this function from HUB if compiled for LMM.
 */
HUBCODE
int getButtons()
{
    int n = 16;
    int result  = -1;
    int reading =  0;
    int pins = 0xFF;
    OUTA |= pins;               //output high to buttons

    while(n--)
    {
        DIRA |= pins;           //output high to buttons
        DIRA ^= pins;           //switch to input
        reading = pins;
        msleep(2);              //wait for RC time
        reading &= INA;          //return button states
        result  &= reading;
    }
    result = ~result & 0xFF;
    return result;
}

/*
 * show the message in persistence of vision style
 */
void message(int index, int delay)
{
    int n = 0;
    int k = 10;

    /* show each char in the phrase */
    for(k = 0; phrase[index][k] != 0; k++)
    {
        for(n = 0; n < 8; n++)
        {
            LEDOUT(fonttab[(int)phrase[index][k]][n]);
            msleep(delay<<2);
        }
    }
}


/*
 * This is the font table. iI was taken from a Spin driver.
 * The syntax is not ANSI C89 compliant.
 * I'm too lazy to convert the numbers to hex.
 */
char fonttab[FONTCHARS][FONTWIDTH] =
{
    {
            0b00000000,  // ........    $00
            0b00000000,  // ........
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b11111111,  // ####....    $01
            0b11111111,  // ####....
            0b11111111,  // ####....
            0b11111111,  // ####....
            0b11111111,  // ........
            0b11111111,  // ........
            0b11111111,  // ........
            0b11111111   // ........
    },{
            0b00000000,  //$02  USED FOR CURSOR
            0b01100000,  //
            0b00110000,  //
            0b00011000,  //
            0b00001100,  //
            0b00000000,  //
            0b00000000,  //
            0b00000000
    },{
            0b11111111,  // ########    $03
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $04
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000   //.####....
    },{
            0b11110000,  // ####....    $05
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000   // ####....
    },{
            0b00001111,  // ....####    $06
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000   // ####....
    },{
            0b11111111,  // ########    $07
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000   // ####....
    },{
            0b00000000,  // ........    $08
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111   // ....####
    },{
            0b11110000,  // ####....    $09
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111   // ....####
    },{
            0b00001111,  // ....####    $0A
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111   // ....####
    },{
            0b11111111,  // ########    $0B
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111   // ....####
    },{
            0b00000000,  // ........    $0C
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111   //.########
    },{
            0b11110000,  // ####....    $0D
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11110000,  // ####....
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111   // ########
    },{
            0b00001111,  // ....####    $0E
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b00001111,  // ....####
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111   // ########
    },{
            0b11111111,  // ########    $0F
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111   // ########
    },{
            0b00000000,  // ........    $10
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b11111111,  // ########
            0b00000000   // ........
    },{
            0b00000000,  // ........    $11
            0b11111111,  // ########
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $12
            0b11111111,  // ########
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000001,  // .......#
            0b00000000   // ........
    },{
            0b00000000,  // ........    $13
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b11111111,  // ########
            0b00000000   // ........
    },{
            0b00000000,  // ........    $14
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b11111111,  // ########
            0b00000000   // ........
    },{
            0b00000000,  // ........    $15
            0b00000000,  // ........
            0b00000000,  // ........
            0b00011111,  // ...#####
            0b00011111,  // ...#####
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000   // ...##...
    },{
            0b00000000,  // ........    $16
            0b00000000,  // ........
            0b00000000,  // ........
            0b11111111,  // ########
            0b11111111,  // ########
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $17
            0b00000000,  // ........
            0b00000000,  // ........
            0b11111000,  // #####...
            0b11111000,  // #####...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000   // ...##...
    },{
            0b00011000,  // ...##...    $18
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011111,  // ...#####
            0b00011111,  // ...#####
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00001100,  // ....##..    $19
            0b00001100,  // ....##..
            0b00001100,  // ....##..
            0b11111100,  // ######..
            0b11111100,  // ######..
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000100,  // .....#..    $1A
            0b00001100,  // ....##..
            0b00011100,  // ...###..
            0b00111100,  // ..####..
            0b00011100,  // ...###..
            0b00001100,  // ....##..
            0b00000100,  // .....#..
            0b00000000   // ........
    },{
            0b00100000,  // ..#.....    $1B
            0b00110000,  // ..##....
            0b00111000,  // ..###...
            0b00111100,  // ..####..
            0b00111000,  // ..###...
            0b00110000,  // ..##....
            0b00100000,  // ..#.....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $1C
            0b00011000,  // ...##...
            0b00111100,  // ..####..
            0b01111110,  // .######.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $1D
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b01111110,  // .######.
            0b00111100,  // ..####..
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $1E
            0b00010000,  // ...#....
            0b00110000,  // ..##....
            0b01111110,  // .######.
            0b01111110,  // .######.
            0b00110000,  // ..##....
            0b00010000,  // ...#....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $1F
            0b00001000,  // ....#...
            0b00001100,  // ....##..
            0b01111110,  // .######.
            0b01111110,  // .######.
            0b00001100,  // ....##..
            0b00001000,  // ....#...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $20  Space
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $21  !
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $22  "
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $23  #
            0b01100110,  // .##..##.
            0b11111111,  // ########
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b11111111,  // ########
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00011000,  // ...##...    $24  $
            0b00111110,  // ..#####.
            0b01100000,  // .##.....
            0b00111100,  // ..####..
            0b00000110,  // .....##.
            0b01111100,  // .#####..
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $25  0b
            0b01100110,  // .##..##.
            0b01101100,  // .##.##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01100110,  // .##..##.
            0b01000110,  // .#...##.
            0b00000000   // ........
    },{
            0b00011100,  // ...###..    $26  &
            0b00110110,  // ..##.##.
            0b00011100,  // ...###..
            0b00111000,  // ..###...
            0b01101111,  // .##.####
            0b01100110,  // .##..##.
            0b00111011,  // ..###.##
            0b00000000   // ........
    },{
            0b00000000,  // ........    $27  //
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $28  (
            0b00001110,  // ....###.
            0b00011100,  // ...###..
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011100,  // ...###..
            0b00001110,  // ....###.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $29  )
            0b01110000,  // .###....
            0b00111000,  // ..###...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00111000,  // ..###...
            0b01110000,  // .###....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $2A  *
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b11111111,  // ########
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $2B  +
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b01111110,  // .######.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $2C  ,
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00110000   // ..##....
    },{
            0b00000000,  // ........    $2D  -
            0b00000000,  // ........
            0b00000000,  // ........
            0b01111110,  // .######.
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $2E  .
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $2F  /
            0b00000110,  // .....##.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01100000,  // .##.....
            0b01000000,  // .#......
            0b00000000   // ........
    },{
            0b00000000,  // ........    $30  0
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01101110,  // .##.###.
            0b01110110,  // .###.##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $31  1
            0b00011000,  // ...##...
            0b00111000,  // ..###...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $32  2
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $33  3
            0b01111110,  // .######.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00001100,  // ....##..
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $34  4
            0b00001100,  // ....##..
            0b00011100,  // ...###..
            0b00111100,  // ..####..
            0b01101100,  // .##.##..
            0b01111110,  // .######.
            0b00001100,  // ....##..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $35  5
            0b01111110,  // .######.
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b00000110,  // .....##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $36  6
            0b00111100,  // ..####..
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $37  7
            0b01111110,  // .######.
            0b00000110,  // .....##.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b00110000,  // ..##....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $38  8
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $39  9
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000110,  // .....##.
            0b00001100,  // ....##..
            0b00111000,  // ..###...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $3A  :
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $3B  ;
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b00000000   // ........
    },{
            0b00000110,  // .....##.    $3C  <
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b00011000,  // ...##...
            0b00001100,  // ....##..
            0b00000110,  // .....##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $3D  =
            0b00000000,  // ........
            0b01111110,  // .######.
            0b00000000,  // ........
            0b00000000,  // ........
            0b01111110,  // .######.
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b01100000,  // .##.....    $3E  >
            0b00110000,  // ..##....
            0b00011000,  // ...##...
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01100000,  // .##.....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $3F  ?
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $40  @
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01101110,  // .##.###.
            0b01101110,  // .##.###.
            0b01100000,  // .##.....
            0b00111110,  // ..#####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $41  A
            0b00011000,  // ...##...
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111110,  // .######.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $42  B
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $43  C
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $44  D
            0b01111000,  // .####...
            0b01101100,  // .##.##..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01101100,  // .##.##..
            0b01111000,  // .####...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $45  E
            0b01111110,  // .######.
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $46  F
            0b01111110,  // .######.
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $47  G
            0b00111110,  // ..#####.
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01101110,  // .##.###.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $48  H
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111110,  // .######.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $49  I
            0b01111110,  // .######.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4A  J
            0b00000110,  // .....##.
            0b00000110,  // .....##.
            0b00000110,  // .....##.
            0b00000110,  // .....##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4B  K
            0b01100110,  // .##..##.
            0b01101100,  // .##.##..
            0b01111000,  // .####...
            0b01111000,  // .####...
            0b01101100,  // .##.##..
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4C  L
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4D  M
            0b01100011,  // .##...##
            0b01110111,  // .###.###
            0b01111111,  // .#######
            0b01101011,  // .##.#.##
            0b01100011,  // .##...##
            0b01100011,  // .##...##
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4E  N
            0b01100110,  // .##..##.
            0b01110110,  // .###.##.
            0b01111110,  // .######.
            0b01111110,  // .######.
            0b01101110,  // .##.###.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $4F  O
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $50  P
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $51  Q
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01101100,  // .##.##..
            0b00110110,  // ..##.##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $52  R
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b01101100,  // .##.##..
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $53  S
            0b00111100,  // ..####..
            0b01100000,  // .##.....
            0b00111100,  // ..####..
            0b00000110,  // .....##.
            0b00000110,  // .....##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $54  T
            0b01111110,  // .######.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $55  U
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $56  V
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $57  W
            0b01100011,  // .##...##
            0b01100011,  // .##...##
            0b01101011,  // .##.#.##
            0b01111111,  // .#######
            0b01110111,  // .###.###
            0b01100011,  // .##...##
            0b00000000   // ........
    },{
            0b00000000,  // ........    $58  X
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $59  Y
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5A  Z
            0b01111110,  // .######.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01100000,  // .##.....
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5B  [
            0b00011110,  // ...####.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011110,  // ...####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5C  backslash
            0b01000000,  // .#......
            0b01100000,  // .##.....
            0b00110000,  // ..##....
            0b00011000,  // ...##...
            0b00001100,  // ....##..
            0b00000110,  // .....##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5D  ]
            0b01111000,  // .####...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b01111000,  // .####...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5E  ^
            0b00001000,  // ....#...
            0b00011100,  // ...###..
            0b00110110,  // ..##.##.
            0b01100011,  // .##...##
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $5F  _
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $60  `
            0b00000001,  // .......#
            0b00000011,  // ......##
            0b00000110,  // .....##.
            0b11001100,  // ##..##..
            0b01111000,  // .####...
            0b00110000,  // ..##....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $61  a
            0b00000000,  // ........
            0b00111100,  // ..####..
            0b00000110,  // .....##.
            0b00111110,  // ..#####.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $62  b
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $63  c
            0b00000000,  // ........
            0b00111100,  // ..####..
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $64  d
            0b00000110,  // .....##.
            0b00000110,  // .....##.
            0b00111110,  // ..#####.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $65  e
            0b00000000,  // ........
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01111110,  // .######.
            0b01100000,  // .##.....
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $66  f
            0b00001110,  // ....###.
            0b00011000,  // ...##...
            0b00111110,  // ..#####.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $67  g
            0b00000000,  // ........
            0b00111110,  // ..#####.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000110,  // .....##.
            0b01111100   // .#####..
    },{
            0b00000000,  // ........    $68  h
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $69  i
            0b00011000,  // ...##...
            0b00000000,  // ........
            0b00111000,  // ..###...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $6A  j
            0b00001100,  // ....##..
            0b00000000,  // ........
            0b00001100,  // ....##..
            0b00001100,  // ....##..
            0b00001100,  // ....##..
            0b00001100,  // ....##..
            0b01111000   // .####...
    },{
            0b00000000,  // ........    $6B  k
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01101100,  // .##.##..
            0b01111000,  // .####...
            0b01101100,  // .##.##..
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $6C  l
            0b00111000,  // ..###...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $6D  m
            0b00000000,  // ........
            0b01100110,  // .##..##.
            0b01111111,  // .#######
            0b01111111,  // .#######
            0b01101011,  // .##.#.##
            0b01100011,  // .##...##
            0b00000000   // ........
    },{
            0b00000000,  // ........    $6E  n
            0b00000000,  // ........
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $6F  o
            0b00000000,  // ........
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $70  p
            0b00000000,  // ........
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01111100,  // .#####..
            0b01100000,  // .##.....
            0b01100000   // .##.....
    },{
            0b00000000,  // ........    $71  q
            0b00000000,  // ........
            0b00111110,  // ..#####.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000110,  // .....##.
            0b00000110   // .....##.
    },{
            0b00000000,  // ........    $72  r
            0b00000000,  // ........
            0b01111100,  // .#####..
            0b01100110,  // .##..##.
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b01100000,  // .##.....
            0b00000000   // ........
    },{
            0b00000000,  // ........    $73  s
            0b00000000,  // ........
            0b00111110,  // ..#####.
            0b01100000,  // .##.....
            0b00111100,  // ..####..
            0b00000110,  // .....##.
            0b01111100,  // .#####..
            0b00000000   // ........
    },{
            0b00000000,  // ........    $74  t
            0b00011000,  // ...##...
            0b01111110,  // .######.
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00001110,  // ....###.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $75  u
            0b00000000,  // ........
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $76  v
            0b00000000,  // ........
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00011000,  // ...##...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $77  w
            0b00000000,  // ........
            0b01100011,  // .##...##
            0b01101011,  // .##.#.##
            0b01111111,  // .#######
            0b00111110,  // ..#####.
            0b00110110,  // ..##.##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $78  x
            0b00000000,  // ........
            0b01100110,  // .##..##.
            0b00111100,  // ..####..
            0b00011000,  // ...##...
            0b00111100,  // ..####..
            0b01100110,  // .##..##.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $79  y
            0b00000000,  // ........
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b01100110,  // .##..##.
            0b00111110,  // ..#####.
            0b00001100,  // ....##..
            0b01111000   // .####...
    },{
            0b00000000,  // ........    $7A  z
            0b00000000,  // ........
            0b01111110,  // .######.
            0b00001100,  // ....##..
            0b00011000,  // ...##...
            0b00110000,  // ..##....
            0b01111110,  // .######.
            0b00000000   // ........
    },{
            0b00000000,  // ........    $7B  {
            0b00011100,  // ...###..
            0b00011000,  // ...##...
            0b01111000,  // .###....
            0b01111000,  // .###....
            0b00011000,  // ...##...
            0b00011100,  // ...###..
            0b00000000   // ........
    },{
            0b00011000,  // ...##...    $7C  |
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000,  // ...##...
            0b00011000   // ...##...
    },{
            0b00000000,  // ........    $7D  }
            0b00111000,  // ..###...
            0b00011000,  // ...##...
            0b00011110,  // ....###.
            0b00011110,  // ....###.
            0b00011000,  // ...##...
            0b00111000,  // ..###...
            0b00000000   // ........
    },{
            0b00000000,  // ........    $7E  ~
            0b00110011,  // ..##..##
            0b01111110,  // .######.
            0b11001100,  // ##..##..
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    },{
            0b00000000,  // ........    $7F
            0b00111000,  // ..###...
            0b01000100,  // .#...#..
            0b01000100,  // .#...#..
            0b00111000,  // ..###...
            0b00000000,  // ........
            0b00000000,  // ........
            0b00000000   // ........
    }
};

/*
    +--------------------------------------------------------------------
      TERMS OF USE: MIT License
    +--------------------------------------------------------------------
    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
    +--------------------------------------------------------------------
*/ 


/* end of file */




